@@ -78,7 +78,7 @@ function task_service {
     elif [ "$OSFAMILY" = "Debian" ]; then
         unitdir=${unitdir_debian}
         defaultsdir=${defaultsdir_debian}
-        sysv_codenames=("squeeze" "wheezy" "lucid" "precise" "trusty" "jessie")
+        sysv_codenames=("squeeze" "wheezy" "lucid" "precise" "trusty")
         if $(echo ${sysv_codenames[@]} | grep -q $CODENAME) ; then
             task install_source_deb_sysv
         else
@@ -110,7 +110,7 @@ function task_install_source_rpm_systemd {
     task postinst_permissions
 }
 
-# Source based install for Debian based setups
+# Source based install for Debian based + sysv setups
 function task_install_source_deb_sysv {
     task preinst_deb
     task install_deb
@@ -248,9 +248,9 @@ function task_sysv_init_deb {
     install -d -m 0755 "${DESTDIR}${rundir}"
 }
 
-# Install the systemd and defaults configuration for Debian.
+# Install the systemd/sysv and defaults configuration for Debian.
 function task_systemd_deb {
-    task defaults_deb
+    task sysv_init_deb
     install -d -m 0755 "${DESTDIR}${unitdir_debian}"
     install -m 0644 ext/debian/puppetdb.service_file "${DESTDIR}${unitdir_debian}/puppetdb.service"
     install -d -m 0755 "${DESTDIR}${tmpfilesdir}"
