@@ -1,6 +1,7 @@
 (ns puppetlabs.puppetdb.jdbc
   "Database utilities"
   (:import (com.zaxxer.hikari HikariDataSource HikariConfig)
+           [java.sql Connection SQLException]
            (java.util.concurrent TimeUnit))
   (:require [clojure.java.jdbc :as sql]
             [clojure.string :as string]
@@ -11,7 +12,8 @@
             [puppetlabs.puppetdb.jdbc.internal :refer [limit-result-set!]]
             [puppetlabs.puppetdb.schema :as pls]
             [schema.core :as s]
-            [clojure.math.numeric-tower :as math]))
+            [clojure.math.numeric-tower :as math]
+            [puppetlabs.i18n.core :refer [trs]]))
 
 (def ^:dynamic *db* nil)
 
@@ -24,28 +26,29 @@
        ~@body)))
 
 (defmacro with-db-transaction [opts & body]
-  `(sql/with-db-transaction [db# (db) ~@opts]
+  `(sql/with-db-transaction [db# (db) (hash-map ~@opts)]
      (binding [*db* db#]
        ~@body)))
 
 (defn do-commands
-  "Calls clojure.jdbc/db-do-commands after adding (jdbc/db) as the
-  first argument."
-  {:arglists '([sql-command & sql-commands]
-               [transaction? sql-command & sql-commands])}
-  [transaction? & commands]
-  (apply sql/db-do-commands *db* transaction? commands))
+  "Runs the given commands in a transaction on the database given
+  by (jdbc/db).  If a command is a vector, it will be converted to a
+  string via (apply str v)."
+  [& commands]
+  (sql/db-do-commands *db* true
+                      (for [c commands]
+                        (if (vector? c)
+                          (apply str c)
+                          c))))
 
 (defn do-prepared
-  "Calls clojure.jdbc/db-do-prepared after adding (jdbc/db) as the
-  first argument."
-  {:arglists '([sql & param-groups]
-               [transaction? sql & param-groups])}
-  [transaction? & remainder]
-  (apply sql/db-do-prepared *db* transaction? remainder))
+  "Executes an optionally parametrized sql expression in a transaction on the
+  database given by (jdbc/db)."
+  [sql & params]
+  (sql/db-do-prepared *db* true (into [sql] params) {:multi? true}))
 
 (defn do-commands-outside-txn [& commands]
-  (let [conn (:connection *db*)
+  (let [^Connection conn (:connection *db*)
         orig (.getAutoCommit conn)]
     (.setAutoCommit conn true)
     (try
@@ -55,29 +58,38 @@
       (finally (.setAutoCommit conn orig)))))
 
 (defn insert!
-  "Calls clojure.jdbc/insert! after adding (jdbc/db) as the first argument."
-  {:arglists '([table row-map :transaction? true :entities identity]
-               [table row-map & row-maps :transaction? true :entities identity]
-               [table col-name-vec col-val-vec
-                & col-val-vecs :transaction? true :entities identity])}
-  [table & options]
-  (apply sql/insert! *db* table options))
+  "Inserts a single row in either map form or lists of columns & values form. The
+  database to use is given by (jdbc/db). Returns a one-element sequence with the
+  inserted row as returned by the database."
+  ([table row]
+   (sql/insert! *db* table row {}))
+  ([table columns values]
+   (sql/insert! *db* table columns values {})))
+
+(defn insert-multi!
+  "Inserts multiple rows in either map form or lists of columns & values form. The
+  database to use is given by (jdbc/db). Returns a sequence with every inserted
+  row as returned by the database."
+  ;; since clojure.java.jdbc will open a connection even when given an empty
+  ;; rows or values sequence, bypass it here if either of those are empty
+  ([table rows]
+   (if (seq rows)
+     (sql/insert-multi! *db* table rows {})
+     ()))
+  ([table columns values]
+   (if (seq values)
+     (sql/insert-multi! *db* table columns values {})
+     ())))
 
 (defn update!
   "Calls clojure.jdbc/update! after adding (jdbc/db) as the first argument."
-  {:arglists '([[table set-map where-clause
-                 & {:keys [entities transaction?]
-                    :or {entities identity transaction? true}}]])}
-  [table set-map where-clause & options]
-  (apply sql/update! *db* table set-map where-clause options))
+  [table set-map where-clause]
+  (sql/update! *db* table set-map where-clause {}))
 
 (defn delete!
   "Calls clojure.jdbc/delete! after adding (jdbc/db) as the first argument."
-  {:arglists '([table where-clause
-                & {:keys [entities transaction?]
-                   :or {entities identity transaction? true}}])}
-  [table where-clause & options]
-  (apply sql/delete! *db* table where-clause options))
+  [table where-clause]
+  (sql/delete! *db* table where-clause {}))
 
 (defn query
   "Calls clojure.jdbc/query after adding (jdbc/db) as the first argument."
@@ -97,7 +109,7 @@
   "Calls clojure.jdbc/db-query-with-resultset after adding (jdbc/db)
    as the first argument. Note that this will hold the whole resultset in memory
    due to the default jdbc fetchsize of 0. If streaming is required, use
-   with-query-results-cursor."
+   call-with-query-rows."
   {:arglists '([[sql-string & params] func]
                [[stmt & params] func]
                [[options-map sql-string & params] func])}
@@ -122,17 +134,76 @@
    #(and (map? %)
          (valid-jdbc-query? (:results-query %)))))
 
-(defn convert-result-arrays
-  "Converts Java and JDBC arrays in a result set using the provided function
-  (eg. vec, set). Values which aren't arrays are unchanged."
-  ([result-set]
-     (convert-result-arrays vec result-set))
-  ([f result-set]
-     (let [convert #(cond
-                     (kitchensink/array? %) (f %)
-                     (isa? (class %) java.sql.Array) (f (.getArray %))
-                     :else %)]
-       (map #(kitchensink/mapvals convert %) result-set))))
+(defn sql-array? [x]
+  (isa? (class x) java.sql.Array))
+
+(defn convert-any-sql-array [x convert]
+  (if-not (sql-array? x)
+    x
+    (convert (.getArray ^java.sql.Array x))))
+
+(defn any-sql-array->vec [x]
+  (convert-any-sql-array x vec))
+
+(defn call-with-query-rows
+  "Calls (f rows), where rows is a lazy sequence of rows generated
+  from within a transaction.  The sequence is backed by an active
+  database cursor which will be closed when f returns.  Cancels the
+  query if f throws an exception.  The option names that correspond to
+  jdbc/result-set-seq options will affect the rows produced as they do
+  for that function.  So for example, when as-arrays? is logically
+  true, the result rows will be vectors, not maps, and the first
+  result row will be a vector of column names."
+  ([query f] (call-with-query-rows query {} f))
+  ([[sql & params]
+    {:keys [as-arrays? identifiers qualifier read-columns] :as opts}
+    f]
+   (with-db-transaction []
+     (with-open [stmt (.prepareStatement ^Connection (:connection *db*) sql)]
+       (doall (map-indexed (fn [i param] (.setObject stmt (inc i) param))
+                           params))
+       (.setFetchSize stmt 500)
+       (with-open [rset (.executeQuery stmt)]
+         (try
+           (f (sql/result-set-seq rset opts))
+           (catch Exception e
+             ;; Cancel the current query
+             (.cancel stmt)
+             (throw e))))))))
+
+
+(defn ^:deprecated call-with-array-converted-query-rows
+  "Calls (f rows), where rows is a lazy sequence of rows generated
+  from within a transaction.  Converts any java.sql.Array
+  or (.isArray (class v)) values to a vector.  The sequence is backed
+  by an active database cursor which will be closed when f returns.
+  Cancels the query if f throws an exception.  The option names that
+  correspond to jdbc/result-set-seq options will affect the rows
+  produced as they do for that function.  So for example, when
+  as-arrays? is logically true, the result rows will be vectors, not
+  maps, and the first result row will be a vector of column names.
+  Note that this function is deprecated.  Please prefer
+  call-with-query-rows, and apply any necessary conversions directly
+  to each row.  (Most columns cannot be arrays.)"
+  ([query f] (call-with-array-converted-query-rows query {} f))
+  ([[sql & params]
+    {:keys [as-arrays? identifiers qualifier read-columns] :as opts}
+    f]
+   (with-db-transaction []
+     (with-open [stmt (.prepareStatement ^Connection (:connection *db*) sql)]
+       (doall (map-indexed (fn [i param] (.setObject stmt (inc i) param))
+                           params))
+       (.setFetchSize stmt 500)
+       (let [fix-vals (if as-arrays?
+                        #(mapv any-sql-array->vec %)
+                        #(kitchensink/mapvals any-sql-array->vec %))]
+         (with-open [rset (.executeQuery stmt)]
+           (try
+             (f (map fix-vals (sql/result-set-seq rset opts)))
+             (catch Exception e
+               ;; Cancel the current query
+               (.cancel stmt)
+               (throw e)))))))))
 
 (defn limited-query-to-vec
   "Take a limit and an SQL query (with optional parameters), and return the
@@ -153,10 +224,11 @@
           ((some-fn string? vector?) query)]
    :post [(or (zero? limit) (<= (count %) limit))]}
   (let [sql-query-and-params (if (string? query) [query] query)]
-    (query-with-resultset sql-query-and-params
-                          #(-> (limit-result-set! limit (sql/result-set-seq %))
-                               convert-result-arrays
-                               vec))))
+    (call-with-query-rows
+     sql-query-and-params
+     (fn [rows]
+       (mapv #(kitchensink/mapvals any-sql-array->vec %)
+             (limit-result-set! limit rows))))))
 
 (defn query-to-vec
   "Take an SQL query and parameters, and return the result of the
@@ -254,21 +326,21 @@
    If there are still retries to perform, returns false."
   [remaining :- s/Int
    current :- s/Int
-   exception]
+   ^SQLException exception]
   (cond
    (zero? remaining)
    (do
-     (log/warn "Caught exception. Last attempt, throwing exception.")
+     (log/warn (trs "Caught exception. Last attempt, throwing exception."))
      (throw exception))
 
    :else
    (do
-     (log/debugf "Caught %s: '%s'. SQL Error code: '%s'. Attempt: %s of %s."
-                 (.getName (class exception))
-                 (.getMessage exception)
-                 (.getSQLState exception)
-                 (inc current)
-                 (+ current remaining))
+     (log/debug (trs "Caught {0}: ''{1}''. SQL Error code: ''{2}''. Attempt: {3} of {4}."
+                     (.getName (class exception))
+                     (.getMessage exception)
+                     (.getSQLState exception)
+                     (inc current)
+                     (+ current remaining)))
      (exponential-sleep! current 1.3)
      false)))
 
@@ -325,33 +397,6 @@
      (fn []
        ~@body)))
 
-(defn with-query-results-cursor
-  "Executes the given parameterized query within a transaction,
-  producing a lazy sequence of rows. The callback `func` is executed
-  on the entire sequence.
-
-  The lazy sequence is backed by an active database cursor, and is thus
-  useful for streaming very large resultsets.
-
-  The cursor is closed when `func` returns. If an exception is thrown,
-  the query is cancelled."
-  [[sql & params] func]
-  (with-db-transaction []
-   (with-open [stmt (.prepareStatement (:connection *db*) sql)]
-     (doseq [[index value] (map-indexed vector params)]
-       (.setObject stmt (inc index) value))
-     (.setFetchSize stmt 500)
-     (with-open [rset (.executeQuery stmt)]
-       (try
-         (-> rset
-             sql/result-set-seq
-             convert-result-arrays
-             func)
-         (catch Exception e
-           ;; Cancel the current query
-           (.cancel stmt)
-           (throw e)))))))
-
 (defn ^:dynamic enable-jmx
   "This function exists to enable starting multiple PuppetDB instances
   inside a single JVM. Starting up a second instance results in a
