@@ -10,7 +10,8 @@
             [puppetlabs.puppetdb.schema :as pls]
             [puppetlabs.puppetdb.utils :as utils]
             [puppetlabs.kitchensink.core :as kitchensink]
-            [schema.core :as s])
+            [schema.core :as s]
+            [puppetlabs.i18n.core :refer [trs]])
   (:import [org.postgresql.util PGobject]))
 
 ;; SCHEMA
@@ -211,6 +212,16 @@
   [field]
   (hcore/raw (format "%s @> ?" field)))
 
+(defn jsonb-regex
+  "Produce a predicate that matches a regex against a nested value and checks
+   the existence of a (presumably) top-level value. The existence check is
+   necessary because -> is not indexable but ?? is. Assumes a GIN index on the
+   column supplied."
+  [column qmarks]
+  (let [delimited-qmarks (str/join "->" qmarks)]
+    (hcore/raw (format "(%s->%s)::text ~ ?::text and %s ?? ?"
+                       column delimited-qmarks column))))
+
 (defn db-serialize
   "Serialize `value` into a form appropriate for querying against a
   serialized database column."
@@ -239,7 +250,7 @@
 (defn vacuum-analyze
   [db]
   (sql/with-db-connection [conn db]
-    (sql/execute! db ["vacuum analyze"] :transaction? false)))
+    (sql/execute! db ["vacuum analyze"] {:transaction? false})))
 
 (defn parse-db-hash
   [db-hash]
@@ -298,7 +309,7 @@
 
 (defn analyze-small-tables
   [small-tables]
-  (log/info "Analyzing small tables")
+  (log/info (trs "Analyzing small tables"))
   (apply jdbc/do-commands-outside-txn
          (map #(str "analyze " %) small-tables)))
 
