@@ -1,6 +1,6 @@
 (ns puppetlabs.puppetdb.utils
   (:require [puppetlabs.kitchensink.core :as kitchensink]
-            [puppetlabs.i18n.core :as i18n]
+            [puppetlabs.i18n.core :refer [trs tru]]
             [clojure.tools.logging :as log]
             [clojure.string :as string]
             [schema.core :as s]
@@ -18,6 +18,20 @@
            [java.nio.charset Charset CharsetEncoder CoderResult StandardCharsets]
            [org.postgresql.util PGobject]))
 
+(defn flush-and-exit [status]
+  "Attempts to flush *out* and *err*, reporting any failures to *err*,
+  if possible, and then invokes (System/exit status)."
+  (let [out-ex (try (flush) nil (catch Exception ex ex))]
+    (when out-ex
+      (try
+        (binding [*out* *err*]
+          (println "stdout flush on exit failed: " out-ex)
+          (catch Exception _ nil)))))
+  (try
+    (binding [*out* *err*] (flush))
+    (catch Exception _ nil))
+  (System/exit status))
+
 (defn jdk6?
   "Returns true when the current JDK version is 1.6"
   []
@@ -36,7 +50,7 @@
   "Returns error message instructing the user to switch to JDK 1.7"
   []
   (attention-msg
-   (i18n/trs "JDK 1.6 is no longer supported. PuppetDB requires JDK 1.7+, currently running: {0}" kitchensink/java-version)))
+   (trs "JDK 1.6 is no longer supported. PuppetDB requires JDK 1.7+, currently running: {0}" kitchensink/java-version)))
 
 (defn println-err
   "Redirects output to standard error before invoking println"
@@ -201,8 +215,8 @@
 
 (defn throw+-cli-error!
   [msg]
-  (throw+ {:type ::cli-error
-           :message msg}))
+  (throw+ {:kind ::cli-error
+           :msg msg}))
 
 (defn validate-cli-base-url!
   "A utility function which will validate the base-url
@@ -217,10 +231,10 @@
   (try+
    (body)
    (catch map? m
-     (println (:message m))
-     (case (kitchensink/without-ns (:type m))
-       :cli-error (System/exit 1)
-       :cli-help (System/exit 0)
+     (println (:msg m))
+     (case (kitchensink/without-ns (:kind m))
+       :cli-error (flush-and-exit 1)
+       :cli-help (flush-and-exit 0)
        (throw+ m)))))
 
 (defn pdb-query-base-url
@@ -295,7 +309,8 @@
 (defn regex-quote
   [s]
   (when (and (string? s) (re-find #"\\E" s))
-    (throw (IllegalArgumentException. "cannot regex-quote strings containing '\\E'")))
+    (throw (IllegalArgumentException.
+            (tru "cannot regex-quote strings containing ''\\E''"))))
   (format "\\Q%s\\E" (str s)))
 
 (defn match-any-of
@@ -406,3 +421,25 @@
          (binding [*out* *err*]
            (println ex#))
          (throw ex#)))))
+
+(def content-encodings->file-extensions
+  {"gzip" "gz"
+   "identity" ""})
+
+(defn content-encoding->file-extension
+  [encoding]
+  (get content-encodings->file-extensions encoding ""))
+
+(def compression-file-extension-schema
+  (->> content-encodings->file-extensions
+       vals
+       (cons "")
+       (apply s/enum)))
+
+(def supported-content-encodings
+  (vec (keys content-encodings->file-extensions)))
+
+(defn strip-nil-values
+  "remove all nil-valued keys from a map"
+  [m]
+  (into {} (filter val m)))
